
unit demineur_u1;

{$IFDEF FPC}
{$MODE DELPHI}
{$ENDIF}

interface

const
  NOMBRE_COLONNES = 8;
  NOMBRE_LIGNES   = 8;
  NOMBRE_MINES    = 8;
  
type
  TMines = array[0..NOMBRE_LIGNES - 1, 0..NOMBRE_COLONNES - 1] of integer;

function NouvellesMines(): TMines;
function DecouvreYX(var m: TMines; const y, x: integer): boolean;
procedure DecouvreTout(var m: TMines);
function ToutesLesMinesTrouvees(const m: TMines): boolean;

implementation

uses
  demineur_u2;

function NouvellesMines(): TMines;
var
  x, y, n: integer;
begin
  for y := 0 to NOMBRE_LIGNES - 1 do
    for x := 0 to NOMBRE_COLONNES - 1 do
      result[y, x] := 0;
  
  n := 0;
  while n < NOMBRE_MINES do
  begin
    x := Random(NOMBRE_COLONNES);
    y := Random(NOMBRE_LIGNES);
    if not EstUneMine(result[y, x]) then
    begin
      PoseMine(result[y, x]);
      Inc(n);
    end;
  end;
  
  for y := 0 to NOMBRE_LIGNES - 1 do
    for x := 0 to NOMBRE_COLONNES - 1 do
    begin
      if EstUneMine(result[y, x]) then
      begin
        if (y > 0) and (x > 0)                                   then Inc(result[y - 1, x - 1]);
        if (y > 0)                                               then Inc(result[y - 1, x    ]);
        if (y > 0) and (x < NOMBRE_COLONNES - 1)                 then Inc(result[y - 1, x + 1]);
        if (x > 0)                                               then Inc(result[y    , x - 1]);
        if (x < NOMBRE_COLONNES - 1)                             then Inc(result[y    , x + 1]);
        if (y < NOMBRE_LIGNES - 1) and (x > 0)                   then Inc(result[y + 1, x - 1]);
        if (y < NOMBRE_LIGNES - 1)                               then Inc(result[y + 1, x    ]);
        if (y < NOMBRE_LIGNES - 1) and (x < NOMBRE_COLONNES - 1) then Inc(result[y + 1, x + 1]);
      end;
      
      Recouvre(result[y, x]);
    end;
end;

function DecouvreYX(var m: TMines; const y, x: integer): boolean;
begin
  result := FALSE;
  
  if not EstCouvert(m[y, x]) then
    exit;
  
  Decouvre(m[y, x]);
  
  if EstUneMine(m[y, x]) then
  begin
    result := TRUE;
    exit;
  end;
  
  if EstUnNombre(m[y, x]) then
    exit;
  
  if (y > 0) and (x > 0)                                   then DecouvreYX(m, y - 1, x - 1);
  if (y > 0)                                               then DecouvreYX(m, y - 1, x    );
  if (y > 0) and (x < NOMBRE_COLONNES - 1)                 then DecouvreYX(m, y - 1, x + 1);
  if (x > 0)                                               then DecouvreYX(m, y    , x - 1);
  if (x < NOMBRE_COLONNES - 1)                             then DecouvreYX(m, y    , x + 1);
  if (y < NOMBRE_LIGNES - 1) and (x > 0)                   then DecouvreYX(m, y + 1, x - 1);
  if (y < NOMBRE_LIGNES - 1)                               then DecouvreYX(m, y + 1, x    );
  if (y < NOMBRE_LIGNES - 1) and (x < NOMBRE_COLONNES - 1) then DecouvreYX(m, y + 1, x + 1);
end;

procedure DecouvreTout(var m: TMines);
var
  x, y: integer;
begin
  for y := 0 to  NOMBRE_LIGNES - 1 do
    for x := 0 to NOMBRE_COLONNES - 1 do
      Decouvre(m[y, x]);
end;

function ToutesLesMinesTrouvees(const m: TMines): boolean;
var
  x, y: integer;
begin
  result := TRUE;
  for y := 0 to NOMBRE_LIGNES - 1 do
    for x := 0 to NOMBRE_COLONNES - 1 do
      if EstCouvert(m[y, x]) and not EstUneMine(m[y, x]) then
        result := FALSE;
end;

end.
